/*
 * i2c.c --- I2C bus interface.
 * $Id$
 */
#pragma MESSAGE DISABLE C1420

#include "derivative.h"
#include "types.h"
#include "i2c.h"

#define SCL_BIT			0x01
#define SDA_BIT			0x02
#define WAIT()			{ asm(nop); asm(nop); asm(nop); asm(nop); }

#define DEVICE_ADDR		0x0d



static void start(void);
static void stop(void);
static UInt8 sendrecv(UInt8 senddata, UInt8 *recvdata, UInt8 ack);



void I2C_Init(void)
{
	// SCL, SDA is input, set Low when output
	PTADD &= ~(SCL_BIT | SDA_BIT);
	PTAD  &= ~(SCL_BIT | SDA_BIT);
}



void I2C_ByteWrite(UInt8 reg, UInt8 data)
{
	UInt8 dummy;

	start();
	sendrecv(DEVICE_ADDR << 1, &dummy, 1);	// device address, R/nW
	sendrecv(reg, &dummy, 1);	// register address
	sendrecv(data, &dummy, 1);	// write data
	stop();
}



UInt8 I2C_ByteRead(UInt8 reg)
{
	UInt8 dummy,readdata;

	start();
	sendrecv(DEVICE_ADDR << 1, &dummy, 1);	// device address, R/nW
	sendrecv(0xb0, &dummy, 1);	// set pointer
	sendrecv(reg, &dummy, 1);	// register address
	stop();

	start();
	sendrecv((DEVICE_ADDR << 1) | 0x01, &dummy, 1);	// device address, R/nW
	sendrecv(0xff, &readdata, 1);	// read data
	stop();

	return readdata;
}



static void start(void)
{
	WAIT();
	PTADD &= ~SCL_BIT;
	WAIT();
	PTADD |= SDA_BIT;
	WAIT();
}



static void stop(void)
{
	WAIT();
	PTADD |= SDA_BIT;
	WAIT();
	PTADD &= ~SCL_BIT;
	WAIT();
	PTADD &= ~SDA_BIT;
	WAIT();
}



/*
 * when send:
 *    outdata = <output data>
 *    ack     = 1
 *    return  : recerved ack (zero or non-zero)
 * when recerve:
 *    outdata = 0xff
 *    ack     = <acknowledge> (1:nack  0:ack)
 *    *indata : recerved data
 */
static UInt8 sendrecv(UInt8 senddata, UInt8 *recvdata, UInt8 ack)
{
	UInt8 i, recvack;
	UInt8 tmprecv = 0;

	for(i=0; i!=8; i++) {
		PTADD |= SCL_BIT;
		WAIT();
		if(senddata & 0x80)
			PTADD &= ~SDA_BIT;
		else
			PTADD |= SDA_BIT;
		senddata <<= 1;
		WAIT();
		PTADD &= ~SCL_BIT;
		WAIT();
		tmprecv <<= 1;
		if(PTAD & SDA_BIT) tmprecv |= 1;
		WAIT();
	}

	PTADD |= SCL_BIT;
	WAIT();
	if(ack)
		PTADD &= ~SDA_BIT;
	else
		PTADD |= SDA_BIT;
	WAIT();
	PTADD &= ~SCL_BIT;
	WAIT();
	recvack = PTAD & SDA_BIT;
	WAIT();
	PTADD |= SCL_BIT;
	WAIT();

	*recvdata = tmprecv;
	return recvack;
}