/*
 * main.c --- Entry point and main loop of Tiny Network Analyzer.
 * $Id$
 */

#include "derivative.h"
#include "types.h"
#include "irq.h"
#include "usb.h"
#include "usb_cdc.h"
#include "command.h"



static void hw_init(void);



void main(void)
{
	USB_EVENT_HANDLER_SET_T usb_event_handlers;

	// initialize hardware
	hw_init();

	// initialize USB fundamental
	usb_event_handlers.usb_reset_handler  = NULL;
	usb_event_handlers.usb_sleep_handler  = NULL;
	usb_event_handlers.usb_resume_handler = NULL;
	usb_event_handlers.usb_ep0_handler    = CDC_EP0Callback;
	USB_Init(&usb_event_handlers);

	// initialize CDC layer
	CDC_Init();
	Command_Init();

	// enable interrupts
	irq_enable();

	// main loop...
	while(1) {
		Command_MainProcess();
		CDC_MainProcess();
	}
}



static void hw_init(void)
{
	// stop WDT
	SOPT1_COPT=0;

	// system configration
	SOPT1 = 0xf3;
	SOPT2 = 0x00;
	SPMSC1 = 0x40;
	SPMSC2 = 0x00;

	// I/O port configuration
	PTAD = 0x00;	// PTA[2] is Low
	PTADD = 0x0c;	// PTA[3:2] is output
	PTAPE = 0xf0;	// PTA[7:4] is pull-up, these pin is not used
	PTASE = 0x00;	// slew rate
	PTADS = 0x00;	// drive strength

	PTBPE = 0x03;	// PTB[0:1] is pull-up, these pin is not used

	/*
	 * clock configulation
	 * NOTE: We assume that the 12MHz external X'tal is connected
	 */
	MCGC2 = 0x36;
	while(!(MCGSC & 0x02));
	MCGC1 = 0x98;
	while((MCGSC & 0x1c ) != 0x08);

	MCGC3 = 0x48;
	while ((MCGSC & 0x48) != 0x48);

	MCGC1 &= 0x3F;
	while((MCGSC & 0x6c) != 0x6c);
}