/*
 * usb.h --- USB function foundation.
 * $Id$
 */

#ifndef _USB_H_
#define _USB_H_

#include "types.h"
#include "usb_config.h"

/* declare Buffer Descriptor Table (BDT) */
#define BDT_BASE				0x1860
#define BDT_CTRL(x)				( *(UInt8*)(BDT_BASE + (3 * (x)) + 0) )
#define BDT_BYTECOUNT(x)		( *(UInt8*)(BDT_BASE + (3 * (x)) + 1) )
#define BDT_ADDR(x)				( *(UInt8*)(BDT_BASE + (3 * (x)) + 2) )

#define BDT_CTRL_OWN			0x80
#define BDT_CTRL_DATA01			0x40
#define BDT_CTRL_KEEP			0x20
#define BDT_CTRL_NINC			0x10
#define BDT_CTRL_DTS			0x08
#define BDT_CTRL_BDTSTALL		0x04
#define BDT_CTRL_BDTKPID		0x3c

/* declare setup packet */
#define USB_SETUP_INDEX_REQTYPE		0
#define USB_SETUP_INDEX_REQ			1
#define USB_SETUP_INDEX_VALUE		2
#define USB_SETUP_INDEX_VALUE_L		2
#define USB_SETUP_INDEX_VALUE_H		3
#define USB_SETUP_INDEX_INDEX		4
#define USB_SETUP_INDEX_INDEX_L		4
#define USB_SETUP_INDEX_INDEX_H		5
#define USB_SETUP_INDEX_LENGTH		6
#define USB_SETUP_INDEX_LENGTH_L	6
#define USB_SETUP_INDEX_LENGTH_H	7

/* declare PID */
#define SETUP_TOKEN					0x0d
#define OUT_TOKEN					0x01
#define IN_TOKEN					0x09

/* declare standard USB request */
#define USBRQ_SET_ADDRESS			0x05
#define USBRQ_GET_DESCRIPTOR		0x06
#define USBRQ_GET_CONFIGURATION		0x08
#define USBRQ_SET_CONFIGURATION		0x09

#define STDD_DEVICE         		0x01
#define STDD_CONFIG         		0x02
#define STDD_STRING         		0x03
#define STDD_INTERFACE      		0x04
#define STDD_ENDPOINT       		0x05

/* declare endpoint types */
#define EP_TYPE_CONTROL 			0x00
#define EP_TYPE_ISO     			0x01
#define EP_TYPE_BULK    			0x02
#define EP_TYPE_IT      			0x03
#define EP_TYPE_DISABLE 			0x04

/* declare device states */
#define USB_DEVICE_STATE_DISABLED		0x01
#define USB_DEVICE_STATE_DEFAULT		0x02
#define USB_DEVICE_STATE_ADDR_PENDING	0x04
#define USB_DEVICE_STATE_ADDRESSED		0x08
#define USB_DEVICE_STATE_CONFIGURED		0x10



typedef void (*USB_EVENT_HANDLER_T)(void);
typedef struct {
	USB_EVENT_HANDLER_T usb_reset_handler;
	USB_EVENT_HANDLER_T usb_sleep_handler;
	USB_EVENT_HANDLER_T usb_resume_handler;
	USB_EVENT_HANDLER_T usb_ep0_handler;
} USB_EVENT_HANDLER_SET_T;



void   USB_Init(USB_EVENT_HANDLER_SET_T *usb_event_handlers);
void   USB_Send(UInt8 ep, const UInt8 *data, UInt16 tr_length, UInt16 req_length);
void   USB_Receive(UInt8 ep, UInt8 *data, UInt16 tr_length);
UInt8  USB_GetState(void);
UInt16 USB_GetDone(UInt8 ep);
UInt8  USB_EPIsBusy(UInt8 ep);

#endif	/* !_USB_H_ */