/*
 * usb_cdc.h --- Handle for the USB CDC virtual COM port.
 * $Id$
 */

#include "types.h"
#include "usb.h"
#include "usb_cdc.h"
#include "usb_config.h"

#define CDCRQ_SET_LINE_CODING           0x20
#define CDCRQ_GET_LINE_CODING           0x21
#define CDCRQ_SET_CONTROL_LINE_STATE    0x22
#define CDCRQ_GET_SERIAL_STATE          0x20

static UInt8 s_rx_buf[CDC_RX_EP_BUFFER_SIZE];
static UInt8 s_rx_length;
static UInt8 s_rx_index;

static UInt8 s_tx_buf[CDC_TX_EP_BUFFER_SIZE];
static UInt8 s_tx_index;

static UInt8 line_coding[7] = { 0x80, 0x25, 0, 0, 0, 0, 8 };	// 9600bps
static const UInt8 uart_state[2] = { 0, 0 };	// fixed



void CDC_Init(void)
{
	s_rx_length = 0;
	s_rx_index = 0;
	s_tx_index = 0;
}



void CDC_MainProcess(void)
{
	if(USB_GetState() != USB_DEVICE_STATE_CONFIGURED)
		// USB is not configured, nothing to do
		return;

	if(!USB_EPIsBusy(CDC_RX_EP_NO) && (s_rx_length <= s_rx_index)) {
		if(s_rx_length == 0) {
			s_rx_length = (UInt8)USB_GetDone(CDC_RX_EP_NO);
			s_rx_index = 0;

			if(s_rx_length)
				goto skip;
		}

		s_rx_length = 0;
		USB_Receive(CDC_RX_EP_NO, s_rx_buf, sizeof(s_rx_buf));
	}

skip:
	if(!USB_EPIsBusy(CDC_TX_EP_NO) && (s_tx_index != 0)) {
		USB_Send(CDC_TX_EP_NO, s_tx_buf, s_tx_index, s_tx_index);
		s_tx_index = 0;
	}
}



void CDC_EP0Callback(void)
{
	UInt8 *pdata = (UInt8 *)USB_EP0_RX_BUF_BASE;
	UInt8  reqtype = pdata[USB_SETUP_INDEX_REQTYPE];
	UInt8  req     = pdata[USB_SETUP_INDEX_REQ];
//	UInt16 value   = ((UInt16)pdata[USB_SETUP_INDEX_VALUE_H] << 8) |
//	                 pdata[USB_SETUP_INDEX_VALUE_L];
	UInt16 index   = ((UInt16)pdata[USB_SETUP_INDEX_INDEX_H] << 8) |
	                 pdata[USB_SETUP_INDEX_INDEX_L];
	UInt16 length  = ((UInt16)pdata[USB_SETUP_INDEX_LENGTH_H] << 8) |
	                 pdata[USB_SETUP_INDEX_LENGTH_L];

	if(index != CMD_IFC_INDEX)
		return;

	if(reqtype == 0xa1) {
		switch(req) {
		case CDCRQ_GET_SERIAL_STATE:
			USB_Send(0, uart_state, 2, length);
			break;

		case CDCRQ_GET_LINE_CODING:
			USB_Send(0, line_coding, 7, length);
			break;

		default:
			break;
		}
	}

	if(reqtype == 0x21) {
		switch(req) {
		case CDCRQ_SET_LINE_CODING:
			USB_Receive(0, line_coding, 7);
			break;

		case CDCRQ_SET_CONTROL_LINE_STATE:
			break;

		default:
			break;
		}
	}
}



UInt8 CDC_IsTxDataReady(void)
{
	return (s_tx_index < sizeof(s_tx_buf)) ? 1 : 0;
}



UInt8 CDC_IsRxDataReady(void)
{
	return (s_rx_length > s_rx_index) ? 1 : 0;
}



void CDC_PutByte(UInt8 c)
{
	if(s_tx_index < sizeof(s_tx_buf))
		s_tx_buf[s_tx_index++] = c;
}



UInt8 CDC_GetByte(void)
{
	if(s_rx_index >= s_rx_length)
		return 0;
 
	return s_rx_buf[s_rx_index++];
}