/*
 * usb_config.h --- Constant table for USB configuration.
 * $Id$
 */

#include "types.h"
#include "usb.h"



/*
 * Device Descriptor
 */
const UInt8 usb_device_descriptor[] = {
/* 0 */		18,				// Size of this descriptor in bytes
/* 1 */		STDD_DEVICE,	// DEVICE descriptor type
/* 2 */		0x01, 0x01,		// USB Spec Release Number in BCD format: 1.1
/* 4 */		2,				// Class Code: CDC Device
/* 5 */		0,				// Subclass code
/* 6 */		0,				// Protocol code
/* 7 */		EP0_BUFFER_SIZE,// Max packet size for EP0
/* 8 */		0xad, 0xde,		// Vendor ID (0xdead = Dead.)
/* 10 */	0x00, 0x00,		// Product ID
/* 12 */	0x00, 0x00,		// Device release number in BCD format
/* 14 */	1,				// Manufacturer string index
/* 15 */	2,				// Product string index
/* 16 */	3,				// Device serial number string index
/* 17 */	1				// Number of possible configurations
/* total 18 bytes */
};



/*
 * Config Descriptor
 */
const UInt8 usb_config_descriptor[] = {
/* 0 */		9,				// Size of this descriptor in bytes
/* 1 */		2,				// CONFIGURATION descriptor type
/* 2 */		66, 0,			// Total length of data for this cfg
/* 4 */		2,				// Number of interfaces in this cfg
/* 5 */		1,				// Index value of this configuration
/* 6 */		0,				// Configuration string index
/* 7 */		0xa0,			// Attributes
/* 8 */		50,				// Max power consumption (2X mA)

/* Interface Descriptor */
/* 9 */		9,				// Size of this descriptor in bytes
/* 10 */	0x04,			// INTERFACE descriptor type
/* 11 */	0,				// Interface Number
/* 12 */	0,				// Alternate Setting Number
/* 13 */	1,				// Number of endpoints in this intf
/* 14 */	0x02,			// Class code
/* 15 */	0x02,			// Subclass code
/* 16 */	0x01,			// Protocol code
/* 17 */	0,				// Interface string index

/* CDC Header Functional Descriptor */
/* 18 */	5,				// Size of this descriptor in bytes
/* 19 */	0x24,
/* 20 */	0x00,
/* 21 */	0x01, 0x10,		// 0x1001

/* CDC Abstract Control Management Functional Descriptor */
/* 23 */	4,				// Size of this descriptor in bytes
/* 24 */	0x24,
/* 25 */	0x02,
/* 26 */	0x00,

/* CDC Union Functional Descriptor */
/* 27 */	5,				// Size of this descriptor in bytes
/* 28 */	0x24,
/* 29 */	0x06,
/* 30 */	0x00,
/* 31 */	0x01,

/* CDC Call Management Functional Descriptor */
/* 32 */	5,				// Size of this descriptor in bytes
/* 33 */	0x24,
/* 34 */	0x01,
/* 35 */	0x00,
/* 36 */	0x01,

/* Endpoint Descriptor [0] */
/* 37 */	7,				// Size of this descriptor in bytes
/* 38 */	0x05,
/* 39 */	0x81,			// EP1: IN
/* 40 */	0x03,			// Interrupt transfer
/* 41 */	EP1_BUFFER_SIZE & 0xff, 0,
/* 43 */	2,

/* Interface Descriptor */
/* 44 */	9,				// Size of this descriptor in bytes
/* 45 */	0x04,			// INTERFACE descriptor type
/* 46 */	1,				// Interface Number
/* 47 */	0,				// Alternate Setting Number
/* 48 */	2,				// Number of endpoints in this intf
/* 49 */	0x0a,			// Class code
/* 50 */	0,				// Subclass code
/* 51 */	0,				// Protocol code
/* 52 */	0,				// Interface string index

/* Endpoint Descriptor [1] */
/* 53 */	7,
/* 54 */	0x05,
/* 55 */	CDC_TX_EP_NO | 0x80,	// EP2: IN
/* 56 */	0x02,			// Bulk transfer
/* 57 */	CDC_TX_EP_BUFFER_SIZE & 0xff, 0,
/* 59 */	0,

/* Endpoint Descriptor [2] */
/* 60 */	7,
/* 61 */	0x05,
/* 62 */	CDC_RX_EP_NO | 0x00,	// EP3: OUT
/* 63 */	0x02,			// Bulk transfer
/* 64 */	CDC_RX_EP_BUFFER_SIZE & 0xff, 0,
/* 65 */	0
/* total 66 bytes */
};

const UInt8 * const ep_descriptor_table[2/*interface*/][2/*endpoint*/] = {
	{ &usb_config_descriptor[37], (const UInt8 *)NULL        },	/* 0 */
	{ &usb_config_descriptor[53], &usb_config_descriptor[60] }	/* 1 */
};

const UInt8 max_interface_num[MAX_CONFIG] = {
	2
};

const UInt8 max_endpoint_num[MAX_CONFIG][2/*interface*/] = {
	{ 1, 2 }	/* config 1 */
};



/*
 * String Descriptor
 */
const UInt8 string_descriptor0[] = {
			4,				// Total size of this strings in bytes
/* 1 */		3,				// STRING descriptor type
/* 2 */		0x09, 0x04
/* total 4 bytes */
};

const UInt8 str_manufacturer[] = {
			24,				// Total size of this strings in bytes
/* 1 */		3,				// STRING descriptor type
/* 2 */		'E', 0, 'l', 0, 'e', 0, 'k', 0,
/* 10 */	't', 0, 'r', 0, 'o', 0, 'n', 0,
/* 18 */	'i', 0, 'k', 0, 'a', 0
/* total 24 bytes */
};

const UInt8 str_config[] = {
			44,				// Total size of this strings in bytes
/* 1 */		3,				// STRING descriptor type
/* 2 */		'D', 0, 'e', 0, 'f', 0, 'a', 0,
/* 10 */	'u', 0, 'l', 0, 't', 0, ' ', 0,
/* 18 */	'c', 0, 'o', 0, 'n', 0, 'f', 0,
/* 26 */	'i', 0, 'g', 0, 'u', 0, 'r', 0,
/* 34 */	'a', 0, 't', 0, 'i', 0, 'o', 0,
/* 42 */	'n', 0
/* total 44 bytes */
};

const UInt8 str_interface1[] = {
			8,				// Total size of this strings in bytes
/* 1 */		3,				// STRING descriptor type
/* 2 */		'C', 0, 'D', 0, 'C', 0
/* total 8 bytes */
};

const UInt8 str_interface2[] = {
			18,				// Total size of this strings in bytes
/* 1 */		3,				// STRING descriptor type
/* 2 */		'C', 0, 'D', 0, 'C', 0, '-', 0,
/* 10 */	'D', 0, 'a', 0, 't', 0, 'a', 0
/* total 18 bytes */
};

const UInt8 str_serail_number[] = {
			20,				// Total size of this strings in bytes
/* 1 */		3,				// STRING descriptor type
/* 2 */		'V', 0, 'e', 0, 'r', 0, '.', 0,
/* 10 */	' ', 0, '1', 0, '.', 0, '0', 0,
/* 18 */	'0', 0
/* total 20 bytes */
};

const UInt8 str_product[] = {
			44,				// Total size of this strings in bytes
/* 1 */		3,				// STRING descriptor type
/* 2 */		'T', 0, 'i', 0, 'n', 0, 'y', 0,
/* 10 */	' ', 0, 'N', 0, 'e', 0, 't', 0,
/* 18 */	'w', 0, 'o', 0, 'r', 0, 'k', 0,
/* 26 */	' ', 0, 'A', 0, 'n', 0, 'a', 0,
/* 34 */	'l', 0, 'y', 0, 'z', 0, 'e', 0,
/* 42 */	'r', 0
/* total 44 bytes */
};

const UInt8 * const usb_string_descriptors[] = {
	string_descriptor0,
	str_manufacturer,
	str_product,
	str_serail_number,
	str_config,
	str_interface1,
	str_interface2
};