/*
 * usb_config.h --- USB configuration data definition.
 * $Id$
 */

#ifndef _USB_CONFIG_H_
#define _USB_CONFIG_H_

#include "types.h"

/* max configuration index */
#define MAX_CONFIG_INDEX		1

/* endpoint numbers */
#define CDC_TX_EP_NO			2
#define CDC_RX_EP_NO			3
#define TOTAL_NUMBER_OF_EP		4

#define CMD_IFC_INDEX			0

/* endpoint buffer size and buffer address */
#define EP0_BUFFER_SIZE			8
#define EP1_BUFFER_SIZE			32
#define CDC_TX_EP_BUFFER_SIZE	32
#define CDC_RX_EP_BUFFER_SIZE	32

#define USB_BDT_BASE			0x1860
#define USB_BUF_BASE			0x1880

#define USB_ALIGN_BUF(x)		( ((x) + 0x000f) & 0xfff0 )
#define USB_GET_EPADR(x)		( (((x) - USB_BDT_BASE) >> 2) & 0xff )

#define USB_EP0_TX_BUF_BASE		USB_BUF_BASE
#define USB_EP0_RX_BUF_BASE		USB_ALIGN_BUF(USB_EP0_TX_BUF_BASE + EP0_BUFFER_SIZE)
#define USB_EP1_BUF_BASE		USB_ALIGN_BUF(USB_EP0_RX_BUF_BASE + EP0_BUFFER_SIZE)
#define USB_CDC_TX_BUF_BASE		USB_ALIGN_BUF(USB_EP1_BUF_BASE + EP1_BUFFER_SIZE)
#define USB_CDC_RX_BUF_BASE		USB_ALIGN_BUF(USB_CDC_TX_BUF_BASE + CDC_TX_EP_BUFFER_SIZE)

/* configuration constant data */
#define MAX_CONFIG				1
const UInt8 *const usb_string_descriptors[];
const UInt8 usb_device_descriptor[];
const UInt8 usb_config_descriptor[];
const UInt8 * const ep_descriptor_table[][2];
const UInt8 max_interface_num[];
const UInt8 max_endpoint_num[][2];

/* descriptor information macros */
#define SELECT_EP_DESCRIPTOR(index, ep)	ep_descriptor_table[(index)][(ep)]
#define MAX_INTERFACE(config)	max_interface_num[(config)-1]
#define MAX_EP(config, iface)	max_endpoint_num[(config)-1][iface]



UInt8 is_ifc_ndx(UInt8 cndx, UInt8 indx, UInt8 iset);
UInt8 is_ep_ndx(UInt8 cndx, UInt8 indx, UInt8 endx);
void  *get_ep_descriptor(UInt8 indx, UInt8 endx);

#endif	/* !_USB_CONFIG_H_ */